﻿using ProjectCleaningCar.Entities;

namespace ProjectCleaningCar.Drawning;
/// <summary>
/// Класс, отвечающий за прорисовку базового объекта-сущности
/// </summary>
public class DrawningTruck
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTruck? EntityTruck { get; protected set; }
    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки машины
    /// </summary>
    protected int? _startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки машины
    /// </summary>
    protected int? _startPosY;
    /// <summary>
    /// Ширина прорисовки машины
    /// </summary>
    private readonly int _drawningTruckWidth = 130;
    /// <summary>
    /// Высота прорисовки машины
    /// </summary>
    private readonly int _drawningTruckHeight = 65;
    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;
    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;
    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningTruckWidth;
    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningTruckHeight;
    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningTruck()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningTruck(int speed, double weight, Color bodyColor) : this()
    {
        EntityTruck = new EntityTruck(speed, weight, bodyColor);
    }
    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningTruckWidth">Ширина прорисовки машины</param>
    /// <param name="drawningTruckHeight">Высота прорисовки машины</param>
    protected DrawningTruck(int drawningTruckWidth, int drawningTruckHeight) : this()
    {
        _drawningTruckWidth = drawningTruckWidth;
        _drawningTruckHeight = drawningTruckHeight;
    } 
    /// <summary>
    /// Новый конструктор
    /// </summary>
    /// <param name="truck"></param>
    public DrawningTruck(EntityTruck? truck) : this()
    {
        if (truck == null) return;
        EntityTruck = new EntityTruck(truck.Speed, truck.Weight, truck.BodyColor);
    }
    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя 
    /// разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {

        if (width < _drawningTruckWidth || height < _drawningTruckHeight) { return false; };
        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX != null || _startPosY != null)
        {
            if (_startPosX + _drawningTruckWidth > _pictureWidth)
            {
                _startPosX = -_drawningTruckWidth + _pictureWidth;
            }
            else if (_startPosX < 0)
            {
                _startPosX = 0;
            }
            if (_startPosY + _drawningTruckHeight > _pictureHeight)
            {
                _startPosY = -_drawningTruckHeight + _pictureHeight;
            }
            else if (_startPosY < 0)
            {
                _startPosY = 0;
            }
        }
        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Кордината Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if (x + _drawningTruckWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningTruckWidth;
        }
        else if (x < 0)
        {
            _startPosX = 0;
        }
        else
        {
            _startPosX = x;
        }

        if (y + _drawningTruckHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningTruckHeight;
        }
        else if (y < 0)
        {
            _startPosY = 0;
        }
        else
        {
            _startPosY = y;
        }

    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }
        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityTruck.Step > 0)
                {
                    _startPosX -= (int)EntityTruck.Step;
                }
                return true;
            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningTruckWidth + EntityTruck.Step < _pictureWidth)
                {
                    _startPosX += (int)EntityTruck.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTruck.Step > 0)
                {
                    _startPosY -= (int)EntityTruck.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningTruckHeight + EntityTruck.Step < _pictureHeight)
                {
                    _startPosY += (int)EntityTruck.Step;
                }
                return true;
            default:
                return false;

        }
    }
    /// <summary>
    /// Отрисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }
        Pen pen = new(Color.Black);
        Brush bodyBrush = new SolidBrush(EntityTruck.BodyColor);
        //основание
        g.FillRectangle(bodyBrush, _startPosX.Value, _startPosY.Value + 35, 120, 10);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 35, 120, 10);
        //
        g.FillRectangle(bodyBrush, _startPosX.Value + 100, _startPosY.Value + 5, 20, 30);
        g.DrawRectangle(pen, _startPosX.Value + 100, _startPosY.Value + 5, 20, 30);
        //колёса
        g.FillEllipse(bodyBrush, _startPosX.Value + 10, _startPosY.Value + 45, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 10, _startPosY.Value + 45, 20, 20);
        g.FillEllipse(bodyBrush, _startPosX.Value + 35, _startPosY.Value + 45, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 35, _startPosY.Value + 45, 20, 20);
        g.FillEllipse(bodyBrush, _startPosX.Value + 97, _startPosY.Value + 45, 20, 20);
        g.DrawEllipse(pen, _startPosX.Value + 97, _startPosY.Value + 45, 20, 20);
    }
}
