﻿using Microsoft.Extensions.Logging;
using ProjectCleaningCar.CollectionGenericObjects;
using ProjectCleaningCar.Drawning;
using ProjectCleaningCar.Exceptions;

namespace ProjectCleaningCar;
/// <summary>
/// Форма работы с компанией и ее коллекцией
/// </summary>
public partial class FormCleaningCarCollection : Form
{

    /// <summary>
	/// Хранилище коллекций
	/// </summary>
	private readonly StorageCollection<DrawningTruck> _storageCollection;

    /// <summary>
    /// Компания
    /// </summary>
    private AbstractCompany? _company = null;

    private readonly ILogger _logger;
    /// <summary>
    /// Конструктор
    /// </summary>
    public FormCleaningCarCollection(ILogger<FormCleaningCarCollection> logger)
    {
        InitializeComponent();
        _storageCollection = new();
        _logger = logger;
    }
    /// <summary>
    /// Выбор компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
    {
        panelCompanyTools.Enabled = false;
    }
    /// <summary>
    /// Добавление грузовика
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAddTruck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        FormTruckConfig form = new();
        form._truckDelegate += SetTruck;
        form.Show();
    }

    /// <summary>
    /// Добавление автомобиля в коллекцию
    /// </summary>
    /// <param name="truck"></param>
    private void SetTruck(DrawningTruck truck)
    {
        try
        {
            if (_company == null || truck == null)
            {
                return;
            }
            if (_company + truck != -1)
            {
                MessageBox.Show("Объект добавлен");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Добавлен объект: {0}", truck.GetDataForSave());
            }
        
        }
        catch (CollectionOverflowException)
        {
            MessageBox.Show("Не удалось добавить объект");
            _logger.LogError("Ошибка: В коллекции превышено допустимое количество");
        }
    }

    /// <summary>
    /// Удаление объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRemoveCar_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(maskedTextBoxPosition.Text) || _company == null)
        {
            return;
        }
        if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
        {
            return;
        }
        int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
        try
        {
            if (_company - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _company.Show();
                _logger.LogInformation("Удалён объект по позиции {0}", pos);
            } else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        catch (PositionOutOfCollectionException) 
        {
            MessageBox.Show($"Ошибка при удалении по позиции {pos}");
            _logger.LogError("Ошибка при удалении по позиции {0}", pos);
        }
        catch (ObjectNotFoundException)
        {
            MessageBox.Show($"Ошибка: Не найден объект по позиции {pos}");
            _logger.LogError("Ошибка: Не найден объект по позиции {0}", pos);
        }

    }
    /// <summary>
    /// Передача объекта в другую форму
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonGoToCheck_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        try
        {
            DrawningTruck? truck = null;
            int counter = 100;
            while (truck == null)
            {
                truck = _company.GetRandomObject();
                counter--;
                if (counter <= 0) break;
            }
            if (truck == null)
            {
                return;
            }
            FormCleaningCar form = new FormCleaningCar();
            form.SetCar = truck;
            form.ShowDialog();
        } catch(ObjectNotFoundException)
        {
            _logger.LogError("Ошибка при передаче объекта на FormCleaningCar");
        } 
    }
    /// <summary>
    /// Перерисовка коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonRefresh_Click(object sender, EventArgs e)
    {
        if (_company == null)
        {
            return;
        }
        pictureBox.Image = _company.Show();
    }

    /// <summary>
    /// Добавление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionAdd_Click(object sender, EventArgs e)
    {
        if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
        {
            MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            return;
        }
        CollectionType collectionType = CollectionType.None;
        if (radioButtonMassive.Checked)
        {
            collectionType = CollectionType.Massive;
        }
        else if (radioButtonList.Checked)
        {
            collectionType = CollectionType.List;
        }
        _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
        RerfreshListBoxItems();
        _logger.LogInformation("Добавлена коллекция: {Collection}  типа: {Type}", textBoxCollectionName.Text, collectionType);
    }

    /// <summary>
    /// Обновление списка в listBoxCollection
    /// </summary>
    private void RerfreshListBoxItems()
    {
        listBoxCollection.Items.Clear();
        for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
        {
            string? colName = _storageCollection.Keys?[i];
            if (!string.IsNullOrEmpty(colName))
            {
                listBoxCollection.Items.Add(colName);
            }
        }
    }

    /// <summary>
    /// Удаление коллекции
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCollectionDel_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }
        if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
        {
            return;
        }
        _storageCollection.DelCollection(listBoxCollection.SelectedItem.ToString());
        RerfreshListBoxItems();
        MessageBox.Show($"Коллекция {textBoxCollectionName.Text} удалена");
        _logger.LogInformation("Коллекция удалена: {0}", textBoxCollectionName.Text);
    }

    /// <summary>
    /// Создание компании
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreateCompany_Click(object sender, EventArgs e)
    {
        if (listBoxCollection.SelectedIndex < 0 || listBoxCollection.SelectedItem == null)
        {
            MessageBox.Show("Коллекция не выбрана");
            return;
        }

        ICollectionGenericObjects<DrawningTruck>? collection = _storageCollection[listBoxCollection.SelectedItem.ToString() ?? string.Empty];
        if (collection == null)
        {
            MessageBox.Show("Коллекция не проинициализирована");
            return;
        }
        switch (comboBoxSelectorCompany.Text)
        {
            case "Автопарк":
                _company = new AutoParkService(pictureBox.Width, pictureBox.Height, collection);
                _logger.LogInformation("Создна компания типа {Company}, коллекция: {Collection}", comboBoxSelectorCompany.Text, textBoxCollectionName.Text);
                _logger.LogInformation("Создана компания на коллекции: {Collection}", textBoxCollectionName.Text);
                break;
        }
        panelCompanyTools.Enabled = true;
        RerfreshListBoxItems();
    }

    /// <summary>
    /// Обработка нажатия "Сохранение"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (saveFileDialog.ShowDialog() == DialogResult.OK)
        {
            try
            {
                _storageCollection.SaveData(saveFileDialog.FileName);
                MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);
            } catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }

    /// <summary>
    /// Обработка нажатия "Загрузка"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
    {
        if (openFileDialog.ShowDialog() == DialogResult.OK)
        {
            
            try
            {
                _storageCollection.LoadData(openFileDialog.FileName);
                MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                foreach (var collection in _storageCollection.Keys)
                {
                    listBoxCollection.Items.Add(collection);
                }
                _logger.LogInformation("Загрузка из файла: {filename}", saveFileDialog.FileName);
                RerfreshListBoxItems();
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Не загрузилось: {ex.Message}", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
    }
}